﻿/// <reference path="../../Resources/TypeDefinitions/jquery/jquery.d.ts"/>
/// <reference path="../../Resources/TypeDefinitions/knockout/knockout.d.ts"/>
/// <reference path="../../rubbish.ts"/>

var BrowseFeedPagePackageViewModel = (function () {
    function BrowseFeedPagePackageViewModel(data, feedName, feedType) {
        this.id = data.Id;
        this.version = data.Version;
        this.title = data.Title || data.Id;
        this.summary = data.Summary || data.Description || '';
        if (this.summary.length > 200)
            this.summary = this.summary.substring(0, 200);

        this.detailsUrl = '/feeds/' + encodeURIComponent(feedName) + '/' + encodeURIComponent(data.Id) + '/' + encodeURIComponent(data.Version);
        this.tags = (data.Tags || '').split(' ').map(function (t) {
            return $.trim(t);
        }).filter(function (t) {
            return t.length > 0;
        }).sort();

        if (data.IconUrl)
            this.iconUrl = "url('" + data.IconUrl + "')";
        else
            this.iconUrl = null;

        if (!data.IsLocalPackage && !data.IsCached)
            this.overlayIconUrl = '/resources/images/icons/package-remote.30.png';
        else
            this.overlayIconUrl = null;

        this.feedType = feedType;
    }
    return BrowseFeedPagePackageViewModel;
})();

var BrowseFeedPageViewModel = (function () {
    function BrowseFeedPageViewModel(data) {
        var _this = this;
        var initalValues = this.parseHash();

        this.feedName = data.feedName;
        this.feedType = data.feedType;
        this.packages = ko.observable(new Array());
        this.totalCount = ko.observable(0);
        this.startIndex = ko.observable(initalValues.skip);
        this.pageSize = ko.observable(20);
        this.prereleaseInclusion = ko.observable(initalValues.prerelease);
        this.searchText = ko.observable(initalValues.searchText);
        this.loading = ko.observable(true);

        this.headingText = ko.computed(function () {
            if (_this.loading())
                return 'Retrieving package information...';

            var currentCount = _this.packages().length;

            var count = _this.totalCount();
            if (count == 0)
                return 'No packages found.';
            if (count == 1)
                return 'Displaying 1 matching package.';

            var start = _this.startIndex() + 1;

            if (start == 1 && count <= _this.pageSize())
                return 'Displaying ' + currentCount + ' matching packages.';

            var text = 'Displaying ' + start + '-' + (start + _this.pageSize());

            if (count > start + _this.pageSize())
                text += ' of ' + count;

            return text;
        });

        this.previousDisabled = ko.computed(function () {
            return _this.startIndex() == 0;
        });

        this.nextDisabled = ko.computed(function () {
            return _this.packages().length < _this.pageSize();
        });

        var handleUpdate = function () {
            return _this.update();
        };

        this.startIndex.subscribe(handleUpdate);
        this.pageSize.subscribe(handleUpdate);
        this.prereleaseInclusion.subscribe(handleUpdate);
        this.searchText.subscribe(function () {
            if (_this.startIndex() != 0)
                _this.startIndex(0);
            else
                _this.update();
        });

        $('form').submit(function () {
            _this.searchText($('.package-search > input').val());
            return false;
        });

        this.connectorError = ko.observable(null);

        this.update();
    }
    BrowseFeedPageViewModel.prototype.previousClick = function () {
        if (!this.previousDisabled()) {
            if (this.startIndex() > this.pageSize())
                this.startIndex(this.startIndex() - this.pageSize());
            else
                this.startIndex(0);
        }
    };

    BrowseFeedPageViewModel.prototype.nextClick = function () {
        if (!this.nextDisabled()) {
            this.startIndex(this.startIndex() + this.pageSize());
        }
    };

    BrowseFeedPageViewModel.prototype.update = function () {
        var _this = this;
        this.loading(true);

        var url = '/nuget/' + encodeURIComponent(this.feedName);

        var includePrerelease = this.prereleaseInclusion() == 'prerelease';

        var searchText = $.trim(this.searchText());
        if (searchText) {
            url += "/Search()/?searchTerm='" + encodeURIComponent(searchText) + "'&targetFramework=''&includePrerelease=" + includePrerelease + '&';
        } else {
            url += "/Packages()/?";
        }

        url += '$filter=' + (includePrerelease ? 'IsAbsoluteLatestVersion' : 'IsLatestVersion');
        url += '&$orderby=DownloadCount%20desc';
        url += '&$top=' + this.pageSize();
        url += '&$skip=' + this.startIndex();
        url += '&$inlinecount=allpages';
        url += '&$format=json';

        $.ajax({
            url: url,
            cache: false,
            success: function (data, textStatus, jqXHR) {
                _this.connectorError(jqXHR.getResponseHeader('x-pgconnectorerror'));
                _this.totalCount(data.d.count);
                _this.packages(data.d.results.map(function (p) {
                    return new BrowseFeedPagePackageViewModel(p, _this.feedName, _this.feedType);
                }));
                _this.loading(false);
                _this.updateHash(searchText, _this.startIndex(), _this.prereleaseInclusion());
            }
        });
    };

    BrowseFeedPageViewModel.prototype.updateHash = function (searchText, skip, prerelease) {
        var hash = '';
        if (searchText)
            hash += 'searchTerm=' + encodeURIComponent(searchText) + '&';
        if (skip)
            hash += 'skip=' + skip + '&';
        if (prerelease != 'stable')
            hash += 'include=' + prerelease + '&';

        hash = hash.substring(0, hash.length - 1);

        var hashIndex = window.location.href.indexOf('#');

        if (hashIndex > 0 || hash)
            window.location.replace('#' + hash);
    };

    BrowseFeedPageViewModel.prototype.parseHash = function () {
        var hash = window.location.hash;
        if (hash && hash[0] == '#')
            hash = hash.substring(1);

        var values = { searchText: '', skip: 0, prerelease: 'stable' };

        var parts = hash.split('&');
        for (var i = 0; i < parts.length; i++) {
            var subParts = parts[i].split('=', 2);
            if (subParts.length >= 2) {
                if (subParts[0] == 'searchTerm')
                    values.searchText = decodeURIComponent(subParts[1]);
                else if (subParts[0] == 'skip')
                    values.skip = parseInt(subParts[1]);
                else if (subParts[0] == 'include')
                    values.prerelease = subParts[1];
            }
        }

        return values;
    };
    return BrowseFeedPageViewModel;
})();
